#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2016 Artoo <artoo@manjaro.org>
#   SPDX-FileCopyrightText: 2017 Philip Müller <philm@manjaro.org>
#   SPDX-FileCopyrightText: 2018 Artoo <artoo@artixlinux.org>
#   SPDX-FileCopyrightText: 2018-2019 Adriaan de Groot <groot@kde.org>
#   SPDX-FileCopyrightText: 2023 José Leiva <anonyzard@disroot.org>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import libcalamares

from libcalamares.utils import target_env_call, warning
from os.path import exists, join


import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext


def pretty_name():
    return _("Configure SysVinit services")


class SysvinitController:
    """
    This is the sysvinit service controller.
    All of its state comes from global storage and the job
    configuration at initialization time.
    """

    def __init__(self):
        self.root = libcalamares.globalstorage.value('rootMountPoint')
        self.initdDir = "/etc/init.d"

        # Translate the entries in the config to the actions passed to sysv-config
        self.services = dict()
        self.services["enable"] = libcalamares.job.configuration.get('services', [])
        self.services["disable"] = libcalamares.job.configuration.get('disable', [])

    def make_failure_description(self, state, name, runlevel):
        """
        Returns a generic "could not <foo>" failure message, specialized
        for the action @p state and the specific service @p name in @p runlevel.
        """
        if state == "enable":
            description = _("Cannot add service {name!s} to run-level {level!s}.")
        elif state == "disable":
            description = _("Cannot remove service {name!s} from run-level {level!s}.")
        else:
            description = _("Unknown service-action <code>{arg!s}</code> for service {name!s} in run-level {level!s}.")

        return description.format(arg=state, name=name, level=runlevel)


    def update(self, state):
        """
        Call update-rc.d for each service listed
        in services for the given @p state. update-rc.d
        is called with @p state as the command as well.
        """

        for svc in self.services.get(state, []):
            if isinstance(svc, str):
                name = svc
                runlevel = ""
                mandatory = False
            else:
                name = svc["name"]
                runlevel = svc.get("runlevel", "")
                mandatory = svc.get("mandatory", False)

            service_path = self.root + self.initdDir + "/" + name

            if exists(service_path):
                if runlevel != "":
                    ec = target_env_call(["update-rc.d", name, state, runlevel])
                else:
                    ec = target_env_call(["update-rc.d", name, state])
                if ec != 0:
                    warning("Cannot {} service {} to {}".format(state, name, runlevel))
                    warning("update-rc.d returned error code {!s}".format(ec))
                    if mandatory:
                        title = _("Cannot modify service")
                        diagnostic = _("<code>update-rc.d {arg!s}</code> call in chroot returned error code {num!s}.").format(arg=state, num=ec)
                        return (title,
                                self.make_failure_description(state, name, runlevel) + " " + diagnostic
                                )
            else:
                warning("Target service {} does not exist in {}.".format(name, self.initdDir))
                if mandatory:
                    title = _("Target service does not exist")
                    diagnostic = _("The path for service {name!s} is <code>{path!s}</code>, which does not exist.").format(name=name, path=service_path)
                    return (title,
                            self.make_failure_description(state, name, runlevel) + " " + diagnostic
                            )


    def run(self):
        """Run the controller
        """

        for state in ("enable", "disable"):
            r = self.update(state)
            if r is not None:
                return r

def run():
    """
    Setup services
    """

    return SysvinitController().run()
